#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Author: Chris Oliver (excid3@gmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

import os, sys
from lib import consts, log

# Correct the current working directory in case app was started from a different directory
#print sys.path
#os.chdir(sys.path[0])
# CAUSES ISSUES WITH PY2EXE

# the name of the gettext domain. because we have our translation files
# not in a global folder this doesn't really matter, setting it to the
# application name is a good idea tough.
LOCALE_PATH = consts.dirLocale
GETTEXT_DOMAIN = consts.appNameShort

# set up the gettext system and locales
import gettext, locale

locale.setlocale(locale.LC_ALL, '')
gettext.bindtextdomain(GETTEXT_DOMAIN, LOCALE_PATH)
gettext.textdomain(GETTEXT_DOMAIN)

# register the gettext function for the whole interpreter as "_"
import __builtin__
__builtin__._ = gettext.gettext

# Parse arguments
if ('--help' in sys.argv) or ('-h' in sys.argv):
    log.info("""Command line parameters:

-h or --help                 displays this message
-v or --version              displays Keryx version number
-c <project name> or
--create <project name>      creates a new project in the default directory
                             with name of <project name>""")
    sys.exit(0)

if ('--version' in sys.argv) or ('-v' in sys.argv):
    log.info('%s %s' % (consts.appName, consts.appVersion))
    sys.exit(0)

if ('--create' in sys.argv) or ('-c' in sys.argv):
    try:
        import platform
        from lib import plugins, project
        try:    name = sys.argv[2]
        except: name = platform.node()
        plugins.load(consts.dirPlugins, '')
        for item in range(0,len(plugins.OSPluginList)):
            if plugins.OSPluginList[item][1].IsOS() == True :
                success = project.createKeryx(name, plugins.OSPluginList[item][0],
                                    plugins.OSPluginList[item][1])
                print success
                if success[0]: log.info(_('Project created successfully'))
                else: raise
            else: raise
    except:
        log.error(_('Unable to create project'))
    sys.exit(0)

# attempt to load interface
#try:
if 1:
    import lib.wxkeryx
    lib.wxkeryx.Start()
#except:
    #log.error(_('Unable to load interface.'))
    #log.info(_('Make sure you have wxPython installed.'))
    #sys.exit(1)
