# -*- coding: utf-8 -*-
#
# Author: Chris Oliver (excid3@gmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

import os, os.path
from gettext import gettext as _

def readConfigValue(config, name, value):
    tmp = config.Read(name)
    if tmp: return tmp
    return value

# Strings
appName      = 'Keryx'
appVersion   = '0.91'
appNameShort = 'keryx'
appFileExt   = '.keryx'
appComments  = _('Bringing Updates Home.')
filterDesc = _('Keryx Project Files') + ' | (*' + appFileExt + ')'

authors = 'Chris Oliver'
email = 'excid3@gmail.com'
artists = 'Chris Oliver'
docwriters = 'Chris Oliver'
translators = ''
copyright = '(C) 2008 Chris Oliver'
description = appName+_(""" is a package manager for computers without internet.""")
license = appName+_(""" is free software; you can redistribute it and/or modify 
it under the terms of the GNU General Public License as published 
by the Free Software Foundation; either version 2 of the License, 
or (at your option) any later version.

Keryx is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
GNU General Public License for more details. You should have 
received a copy of the GNU General Public License along with 
Keryx; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA  02111-1307  USA""")

welcome_message = _('Welcome to Keryx')

config="""# Keryx config file\n
\n
LogDir=\n
LocaleDir=\n
PackagesDir=\n
ProjectsDir=\n
ThemesDir=\n
CurrentTheme=\n"""

# URLs
urlHomepage = 'http://keryx.betaserver.org'
urlHelp = 'http://keryx.betaserver.org/forum'
urlTranslate = 'https://translations.launchpad.net/keryx'
urlBug = 'https://bugs.launchpad.net/keryx'
urlDonate = 'http://keryx.betaserver.org/#donate'

# Directories
dirLog = 'logs'
dirLocale = 'i18n'
dirPackages = 'packages'
dirPlugins = 'plugins'
dirProjects = 'projects'
dirPixmaps = 'pixmaps'
dirThemes = 'themes'
dirThemeDefault = 'default'
dirCurrentTheme = dirThemeDefault

# Files
fileLog   = os.path.join(dirLog, 'log')

# Filetypes
wildcard = '%s (*.keryx)|*.keryx|' \
           '%s (*.*)|*.*' % (_("Keryx project"), _("All files"))
wildcard_plugin = '%s (*.py)|*.py|' \
                  '%s (*.*)|*.*' % (_("Python File"), _("All files"))

# Directories
dirLog = os.path.join(os.getcwd(),'logs')
dirLocale = os.path.join(os.getcwd(), 'i18n')
dirPackages = os.path.join(os.getcwd(), 'packages')
dirPlugins = os.path.join(os.getcwd(),'plugins')
dirProjects = os.path.join(os.getcwd(),'projects')
dirPixmaps = os.path.join(os.getcwd(), 'pixmaps')
dirThemes = os.path.join(dirPixmaps, 'themes')
dirThemeDefault = os.path.join(dirThemes, 'default')
dirCurrentTheme = dirThemeDefault

# Images
fileIco = os.path.join(dirPixmaps,'keryx.ico')
fileLogo = os.path.join(dirPixmaps,'keryx.png')
file_config = os.path.join(os.getcwd(), 'config.ini')

# Import settings
try:
    import log, wx
    
    if os.path.exists(file_config):
        log.info(_('Loading') + " " + file_config)
        cfg = wx.FileConfig(localFilename=file_config, style=wx.CONFIG_USE_LOCAL_FILE)
        dirLog = readConfigValue(cfg, 'LogDir', dirLog)
        dirLocale = readConfigValue(cfg, 'LocaleDir', dirLocale)
        dirPackages = readConfigValue(cfg, 'PackagesDir', dirPackages)
        dirPlugins = readConfigValue(cfg, 'PluginsDir', dirPlugins)
        dirProjects = readConfigValue(cfg, 'ProjectsDir', dirProjects)
        dirThemes = readConfigValue(cfg, 'ThemesDir', dirThemes)
        dirCurrentTheme = readConfigValue(cfg, 'CurrentTheme', dirCurrentTheme)
except: pass

dirLog = os.path.abspath(os.path.join(os.getcwd(), dirLog)) 
dirLocale = os.path.abspath(os.path.join(os.getcwd(), dirLocale))
dirPackages = os.path.abspath(os.path.join(os.getcwd(), dirPackages))
dirPlugins = os.path.abspath(os.path.join(os.getcwd(), dirPlugins))
dirProjects = os.path.abspath(os.path.join(os.getcwd(), dirProjects))
dirThemes = os.path.abspath(os.path.join(os.getcwd(), dirThemes))
dirCurrentTheme = os.path.abspath(os.path.join(dirThemes, dirCurrentTheme))

# Load theme images
icon_download_package = os.path.join(dirThemeDefault, 'download_package.png')
icon_package = os.path.join(dirThemeDefault, 'package.png')
icon_new = os.path.join(dirThemeDefault, 'new.png')
icon_open = os.path.join(dirThemeDefault, 'open.png')
icon_download = os.path.join(dirThemeDefault, 'download.png')
icon_arrow_up = os.path.join(dirThemeDefault, 'arrow_up.png')
icon_arrow_down = os.path.join(dirThemeDefault, 'arrow_down.png')
icon_uptodate = os.path.join(dirThemeDefault, 'uptodate.png')
icon_update = os.path.join(dirThemeDefault, 'update.png')
icon_error = os.path.join(dirThemeDefault, 'error.png')
icon_refresh = os.path.join(dirThemeDefault, 'refresh.png')
icon_project_details = os.path.join(dirThemeDefault, 'project_details.png')
icon_layout = os.path.join(dirThemeDefault, 'layout.png')
icon_plugin = os.path.join(dirThemeDefault, 'plugin.png')
icon_plugin_disable = os.path.join(dirThemeDefault, 'plugin_disabled.png')
icon_plugin_add = os.path.join(dirThemeDefault, 'plugin_add.png')
icon_close = os.path.join(dirThemeDefault, 'close.png')
icon_quit = os.path.join(dirThemeDefault, 'quit.png')
icon_options = os.path.join(dirThemeDefault, 'options.png')
icon_home = os.path.join(dirThemeDefault, 'home.png')
icon_help = os.path.join(dirThemeDefault, 'help.png')
icon_translate = os.path.join(dirThemeDefault, 'translate.png')
icon_bug_report = os.path.join(dirThemeDefault, 'bug.png')
icon_donate = os.path.join(dirThemeDefault, 'donate.png')
icon_about = os.path.join(dirThemeDefault, 'about.png')
icon_find = os.path.join(dirThemeDefault, 'find.png')

#color_update = "light blue"
#color_uptodate = wx.Color(150, 235, 140)
#color_error = wx.Color(255,118,106)
