# -*- coding: utf-8 -*-
#
# Author: Chris Oliver (excid3@gmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

import consts, log, os.path, sys, re

OSPluginList = []
InterfacePluginList = []
app = None

class pluginBase:
    """ Used to make sure plugins have necessary functions """
    def __init__(self):                pass
    def IsOS(self):                    pass
    def cleanup(self):                 pass
    def createProject(self):           pass
    def getWidgets(self, instance):    self.app = instance
    def loadProject(self):             pass
    def loadLocalPackageList(self):    pass
    def loadInternetPackageList(self): pass
    def downloadPackages(self):        pass

def load(dir, instance):
    """ Enumerates a list of plugins in a directory """
    app = instance

    # Make sure dir exists
    if not os.path.isdir(dir):
        log.info(_('Cannot find ') + consts.dirPlugins + _(', exiting.'))
        return

    # Try to load each file in plugins dir
    sys.path.insert(0, dir)
    for file in os.listdir(dir):
        if file.endswith('.py'):
            status = initialize(file)
            if status != None: # Plugin loaded correctly
                status[1].getWidgets(instance) # Give all plugins widget tree
                if status[2] == 'OS':          OSPluginList.append(status)
                elif status[2] == 'Interface': InterfacePluginList.append(status)

def initialize(file):
    """ Initializes plugin given filename and returns its instance """
    try:
        plugin = __import__(re.sub('\.pyc?$', '', file))
        if not hasattr(plugin, 'PLUGIN_NAME'): return
        name = getattr(plugin, 'PLUGIN_NAME')
        instance = getattr(plugin, name)()
        ptype = getattr(plugin, 'PLUGIN_TYPE')
        log.info(_('Plugin loaded: ') + file)
        return [name, instance, ptype]
    except:
        log.error(file + _(' failed to load.'))
        return
